@@ -156,14 +156,18 @@ class ListenersController(base.BaseController):
                     value=headers,
                     option=('%s protocol listener.' % listener_protocol))
 
-    def _validate_cidr_compatible_with_vip(self, vip, allowed_cidrs):
+    def _validate_cidr_compatible_with_vip(self, vips, allowed_cidrs):
         for cidr in allowed_cidrs:
-            # Check if CIDR IP version matches VIP IP version
-            if common_utils.is_cidr_ipv6(cidr) != common_utils.is_ipv6(vip):
-                msg = _("CIDR %(cidr)s IP version incompatible with VIP "
-                        "%(vip)s IP version.")
+            for vip in vips:
+                # Check if CIDR IP version matches VIP IP version
+                if (common_utils.is_cidr_ipv6(cidr) ==
+                        common_utils.is_ipv6(vip)):
+                    break
+            else:
+                msg = _("CIDR %(cidr)s IP version incompatible with all VIPs "
+                        "%(vips)s IP version.")
                 raise exceptions.ValidationException(
-                    detail=msg % {'cidr': cidr, 'vip': vip})
+                    detail=msg % {'cidr': cidr, 'vips': vips})
 
     def _validate_create_listener(self, lock_session, listener_dict):
         """Validate listener for wrong protocol or duplicate listeners
@@ -172,6 +176,11 @@ class ListenersController(base.BaseController):
         """
         listener_protocol = listener_dict.get('protocol')
 
+        _can_tls_offload = (
+            listener_protocol == constants.PROTOCOL_TERMINATED_HTTPS or
+            (listener_protocol == constants.PROTOCOL_PROMETHEUS and
+             self._has_tls_container_refs(listener_dict)))
+
         if listener_dict and listener_dict.get('insert_headers'):
             self._validate_insert_headers(
                 listener_dict['insert_headers'].keys(), listener_protocol)
@@ -186,17 +195,25 @@ class ListenersController(base.BaseController):
 
         # Check for TLS disabled
         if (not CONF.api_settings.allow_tls_terminated_listeners and
-                listener_protocol == constants.PROTOCOL_TERMINATED_HTTPS):
-            raise exceptions.DisabledOption(
-                value=constants.PROTOCOL_TERMINATED_HTTPS, option='protocol')
+                _can_tls_offload):
+            if listener_protocol == constants.PROTOCOL_PROMETHEUS:
+                msg = f"{listener_protocol} with TLS"
+            else:
+                msg = listener_protocol
+            raise exceptions.DisabledOption(value=msg, option='protocol')
 
-        # Check for certs when not TERMINATED_HTTPS
-        if (listener_protocol != constants.PROTOCOL_TERMINATED_HTTPS and
+        # Check for PROMETHEUS listeners disabled
+        if (not CONF.api_settings.allow_prometheus_listeners and
+                listener_protocol == lib_consts.PROTOCOL_PROMETHEUS):
+            raise exceptions.DisabledOption(
+                value=lib_consts.PROTOCOL_PROMETHEUS, option='protocol')
+
+        # Check for certs when not TLS offload capable
+        if (not _can_tls_offload and
                 self._has_tls_container_refs(listener_dict)):
             raise exceptions.ValidationException(detail=_(
-                "Certificate container references are only allowed on "
-                "%s protocol listeners.") %
-                constants.PROTOCOL_TERMINATED_HTTPS)
+                "Certificate container references are not allowed on "
+                "%s protocol listeners.") % listener_protocol)
 
         # Make sure a base certificate exists if specifying a client ca
         if (listener_dict.get('client_ca_tls_certificate_id') and
@@ -206,14 +223,17 @@ class ListenersController(base.BaseController):
                 "An SNI or default certificate container reference must "
                 "be provided with a client CA container reference."))
 
-        # Make sure a certificate container is specified for TERMINATED_HTTPS
-        if (listener_protocol == constants.PROTOCOL_TERMINATED_HTTPS and
+        # Make sure a certificate container is specified for TLS protocols
+        if (_can_tls_offload and
             not (listener_dict.get('tls_certificate_id') or
                  listener_dict.get('sni_containers'))):
+            if listener_protocol == constants.PROTOCOL_PROMETHEUS:
+                msg = f"{listener_protocol} with TLS"
+            else:
+                msg = listener_protocol
             raise exceptions.ValidationException(detail=_(
                 "An SNI or default certificate container reference must "
-                "be provided for %s protocol listeners.") %
-                constants.PROTOCOL_TERMINATED_HTTPS)
+                "be provided for %s protocol listeners.") % msg)
 
         # Make sure we have a client CA cert if they enable client auth
         if (listener_dict.get('client_authentication') !=
@@ -290,12 +310,13 @@ class ListenersController(base.BaseController):
         # Validate allowed CIDRs
         allowed_cidrs = listener_dict.get('allowed_cidrs', []) or []
         lb_id = listener_dict.get('load_balancer_id')
-        vip_db = self.repositories.vip.get(
-            lock_session, load_balancer_id=lb_id)
-        vip_address = vip_db.ip_address
-        self._validate_cidr_compatible_with_vip(vip_address, allowed_cidrs)
+        lb_db = self.repositories.load_balancer.get(
+            lock_session, id=lb_id)
+        vip_addresses = [lb_db.vip.ip_address]
+        vip_addresses.extend([vip.ip_address for vip in lb_db.additional_vips])
+        self._validate_cidr_compatible_with_vip(vip_addresses, allowed_cidrs)
 
-        if listener_protocol == constants.PROTOCOL_TERMINATED_HTTPS:
+        if _can_tls_offload:
             # Validate TLS version list
             validate.check_tls_version_list(listener_dict['tls_versions'])
             # Validate TLS versions against minimum
@@ -357,6 +378,10 @@ class ListenersController(base.BaseController):
                 listener.to_dict(render_unsets=True), None)
 
             if listener_dict['default_pool_id']:
+                if (listener_dict.get('protocol') ==
+                        lib_consts.PROTOCOL_PROMETHEUS):
+                    raise exceptions.ListenerNoChildren(
+                        protocol=lib_consts.PROTOCOL_PROMETHEUS)
                 self._validate_pool(context.session, load_balancer_id,
                                     listener_dict['default_pool_id'],
                                     listener.protocol)
@@ -400,6 +425,9 @@ class ListenersController(base.BaseController):
             listener_dict, load_balancer_id)
         l7policies = listener_dict.pop('l7policies', l7policies)
         if listener_dict.get('default_pool_id'):
+            if listener_dict.get('protocol') == lib_consts.PROTOCOL_PROMETHEUS:
+                raise exceptions.ListenerNoChildren(
+                    protocol=lib_consts.PROTOCOL_PROMETHEUS)
             self._validate_pool(lock_session, load_balancer_id,
                                 listener_dict['default_pool_id'],
                                 listener_dict['protocol'])
@@ -408,6 +436,12 @@ class ListenersController(base.BaseController):
 
         # Now create l7policies
         new_l7ps = []
+
+        if (listener_dict.get('protocol') == lib_consts.PROTOCOL_PROMETHEUS and
+                l7policies):
+            raise exceptions.ListenerNoChildren(
+                protocol=lib_consts.PROTOCOL_PROMETHEUS)
+
         for l7p in l7policies:
             l7p['project_id'] = db_listener.project_id
             l7p['load_balancer_id'] = load_balancer_id
@@ -426,6 +460,11 @@ class ListenersController(base.BaseController):
         return db_listener
 
     def _validate_listener_PUT(self, listener, db_listener):
+        _can_tls_offload = (
+            db_listener.protocol == constants.PROTOCOL_TERMINATED_HTTPS or
+            (db_listener.protocol == constants.PROTOCOL_PROMETHEUS and
+             self._has_tls_container_refs(listener.to_dict())))
+
         # TODO(rm_work): Do we need something like this? What do we do on an
         # empty body for a PUT?
         if not listener:
@@ -440,17 +479,16 @@ class ListenersController(base.BaseController):
                 "%s protocol listener does not support TLS or header "
                 "insertion.") % db_listener.protocol)
 
-        # Check for certs when not TERMINATED_HTTPS
-        if (db_listener.protocol != constants.PROTOCOL_TERMINATED_HTTPS and
+        # Check for certs when not TLS offload capable
+        if (not _can_tls_offload and
                 self._has_tls_container_refs(listener.to_dict())):
             raise exceptions.ValidationException(detail=_(
-                "Certificate container references are only allowed on "
-                "%s protocol listeners.") %
-                constants.PROTOCOL_TERMINATED_HTTPS)
+                "Certificate container references are not allowed on "
+                "%s protocol listeners.") % db_listener.protocol)
 
         # Make sure we have a client CA cert if they enable client auth
-        if ((listener.client_authentication != wtypes.Unset and
-             listener.client_authentication != constants.CLIENT_AUTH_NONE) and
+        if (listener.client_authentication not in
+                (wtypes.Unset, constants.CLIENT_AUTH_NONE) and
             not (db_listener.client_ca_tls_certificate_id or
                  listener.client_ca_tls_container_ref)):
             raise exceptions.ValidationException(detail=_(
@@ -492,9 +530,13 @@ class ListenersController(base.BaseController):
 
         # Validate allowed CIDRs
         if (listener.allowed_cidrs and listener.allowed_cidrs != wtypes.Unset):
-            vip_address = db_listener.load_balancer.vip.ip_address
+            vip_addresses = [db_listener.load_balancer.vip.ip_address]
+            vip_addresses.extend(
+                [vip.ip_address
+                 for vip in db_listener.load_balancer.additional_vips]
+            )
             self._validate_cidr_compatible_with_vip(
-                vip_address, listener.allowed_cidrs)
+                vip_addresses, listener.allowed_cidrs)
 
         # Check TLS cipher prohibit list
         if listener.tls_ciphers:
@@ -571,6 +613,9 @@ class ListenersController(base.BaseController):
         self._set_default_on_none(listener)
 
         if listener.default_pool_id:
+            if db_listener.protocol == lib_consts.PROTOCOL_PROMETHEUS:
+                raise exceptions.ListenerNoChildren(
+                    protocol=lib_consts.PROTOCOL_PROMETHEUS)
             self._validate_pool(context.session, load_balancer_id,
                                 listener.default_pool_id, db_listener.protocol)
 
