@@ -41,7 +41,9 @@ LOG = logging.getLogger(__name__)
 #  to instance" will be logged as usual.
 def retryMaskFilter(record):
     if record.exc_info is not None and isinstance(
-            record.exc_info[1], driver_exc.AmpConnectionRetry):
+            record.exc_info[1], (
+                driver_exc.AmpConnectionRetry,
+                exceptions.ComputeWaitTimeoutException)):
         return False
     return True
 
@@ -102,8 +104,9 @@ class ControllerWorker(object):
         if CONF.task_flow.jobboard_enabled:
             self.services_controller.run_poster(func, *args, **kwargs)
         else:
+            store = kwargs.pop('store', None)
             tf = self.tf_engine.taskflow_load(
-                func(*args), **kwargs)
+                func(*args, **kwargs), store=store)
             with tf_logging.DynamicLoggingListener(tf, log=LOG):
                 tf.run()
 
