@@ -73,24 +73,38 @@ class BaseNeutronDriver(base.AbstractNetworkDriver):
 
     def _port_to_vip(self, port, load_balancer, octavia_owned=False):
         fixed_ip = None
+        additional_ips = []
         for port_fixed_ip in port.fixed_ips:
-            if port_fixed_ip.subnet_id == load_balancer.vip.subnet_id:
+            if (not fixed_ip and
+                    port_fixed_ip.subnet_id == load_balancer.vip.subnet_id):
                 fixed_ip = port_fixed_ip
-                break
+            else:
+                additional_ips.append(port_fixed_ip)
         if fixed_ip:
-            return data_models.Vip(ip_address=fixed_ip.ip_address,
-                                   subnet_id=fixed_ip.subnet_id,
-                                   network_id=port.network_id,
-                                   port_id=port.id,
-                                   load_balancer=load_balancer,
-                                   load_balancer_id=load_balancer.id,
-                                   octavia_owned=octavia_owned)
-        return data_models.Vip(ip_address=None, subnet_id=None,
-                               network_id=port.network_id,
-                               port_id=port.id,
-                               load_balancer=load_balancer,
-                               load_balancer_id=load_balancer.id,
-                               octavia_owned=octavia_owned)
+            primary_vip = data_models.Vip(ip_address=fixed_ip.ip_address,
+                                          subnet_id=fixed_ip.subnet_id,
+                                          network_id=port.network_id,
+                                          port_id=port.id,
+                                          load_balancer=load_balancer,
+                                          load_balancer_id=load_balancer.id,
+                                          octavia_owned=octavia_owned)
+        else:
+            primary_vip = data_models.Vip(ip_address=None, subnet_id=None,
+                                          network_id=port.network_id,
+                                          port_id=port.id,
+                                          load_balancer=load_balancer,
+                                          load_balancer_id=load_balancer.id,
+                                          octavia_owned=octavia_owned)
+        additional_vips = [
+            data_models.AdditionalVip(
+                ip_address=add_fixed_ip.ip_address,
+                subnet_id=add_fixed_ip.subnet_id,
+                network_id=port.network_id,
+                port_id=port.id,
+                load_balancer=load_balancer,
+                load_balancer_id=load_balancer.id)
+            for add_fixed_ip in additional_ips]
+        return primary_vip, additional_vips
 
     def _nova_interface_to_octavia_interface(self, compute_id, nova_interface):
         fixed_ips = [utils.convert_fixed_ip_dict_to_model(fixed_ip)
@@ -108,11 +122,11 @@ class BaseNeutronDriver(base.AbstractNetworkDriver):
                                         port_id=port['id'],
                                         fixed_ips=fixed_ips)
 
-    def _add_allowed_address_pair_to_port(self, port_id, ip_address):
+    def _add_allowed_address_pairs_to_port(self, port_id, ip_address_list):
         aap = {
             'port': {
                 'allowed_address_pairs': [
-                    {'ip_address': ip_address}
+                    {'ip_address': ip} for ip in ip_address_list
                 ]
             }
         }
@@ -275,3 +289,35 @@ class BaseNeutronDriver(base.AbstractNetworkDriver):
 
     def get_network_ip_availability(self, network):
         return self._get_resource('network_ip_availability', network.id)
+
+    def plug_fixed_ip(self, port_id, subnet_id, ip_address=None):
+        port = self.get_port(port_id).to_dict(recurse=True)
+        fixed_ips = port['fixed_ips']
+
+        new_fixed_ip_dict = {'subnet_id': subnet_id}
+        if ip_address:
+            new_fixed_ip_dict['ip_address'] = ip_address
+
+        fixed_ips.append(new_fixed_ip_dict)
+
+        body = {'port': {'fixed_ips': fixed_ips}}
+        try:
+            updated_port = self.neutron_client.update_port(port_id, body)
+            return utils.convert_port_dict_to_model(updated_port)
+        except Exception as e:
+            raise base.NetworkException(str(e))
+
+    def unplug_fixed_ip(self, port_id, subnet_id):
+        port = self.get_port(port_id)
+        fixed_ips = [
+            fixed_ip.to_dict()
+            for fixed_ip in port.fixed_ips
+            if fixed_ip.subnet_id != subnet_id
+        ]
+
+        body = {'port': {'fixed_ips': fixed_ips}}
+        try:
+            updated_port = self.neutron_client.update_port(port_id, body)
+            return utils.convert_port_dict_to_model(updated_port)
+        except Exception as e:
+            raise base.NetworkException(str(e))
