@@ -37,105 +37,6 @@ class TestException(Exception):
         return repr(self.value)
 
 
-class TestSpareCheck(base.TestCase):
-    FAKE_CNF_SPAR1 = 5
-    FAKE_CUR_SPAR1 = 2
-    FAKE_CNF_SPAR2 = 3
-    FAKE_CUR_SPAR2 = 3
-
-    def setUp(self):
-        super().setUp()
-        self.CONF = self.useFixture(oslo_fixture.Config(cfg.CONF))
-        self.CONF.config(group="api_settings",
-                         default_provider_driver='amphora')
-        self.spare_amp = house_keeping.SpareAmphora()
-        self.amp_repo = mock.MagicMock()
-        self.az_repo = mock.MagicMock()
-        self.cw = mock.MagicMock()
-
-        self.spare_amp.amp_repo = self.amp_repo
-        self.spare_amp.az_repo = self.az_repo
-        self.spare_amp.cw = self.cw
-
-    @mock.patch('octavia.db.api.get_session')
-    def test_spare_check_diff_count(self, session):
-        """When spare amphora count does not meet the requirement."""
-        session.return_value = session
-        self.CONF.config(group="house_keeping",
-                         spare_amphora_pool_size=self.FAKE_CNF_SPAR1)
-        self.amp_repo.get_spare_amphora_count.return_value = (
-            self.FAKE_CUR_SPAR1)
-        self.az_repo.get_all.return_value = [], None
-        self.spare_amp.spare_check()
-        self.assertTrue(self.amp_repo.get_spare_amphora_count.called)
-        DIFF_CNT = self.FAKE_CNF_SPAR1 - self.FAKE_CUR_SPAR1
-
-        self.assertEqual(DIFF_CNT, self.cw.create_amphora.call_count)
-
-    @mock.patch('octavia.db.api.get_session')
-    def test_spare_check_diff_count_multi_az(self, session):
-        """When spare amphora count does not meet the requirement.
-
-        Tests when multiple availabilty zones active
-        """
-        session.return_value = session
-        self.CONF.config(group="house_keeping",
-                         spare_amphora_pool_size=self.FAKE_CNF_SPAR1)
-        az1 = mock.Mock()
-        az1.name = 'az1'
-        az2 = mock.Mock()
-        az2.name = 'az2'
-        self.az_repo.get_all.return_value = [az1, az2], None
-        self.amp_repo.get_spare_amphora_count.return_value = (
-            self.FAKE_CUR_SPAR1)
-        self.az_repo.get_availability_zone_metadata_dict().get.side_effect = (
-            az1.name, az2.name)
-        self.spare_amp.spare_check()
-
-        calls = [mock.call(session, availability_zone=az1.name,
-                           check_booting_amphora=False),
-                 mock.call(session, availability_zone=az2.name,
-                           check_booting_amphora=False)]
-        self.amp_repo.get_spare_amphora_count.assert_has_calls(calls,
-                                                               any_order=True)
-
-        # 2 AZs so twice as many calls
-        DIFF_CNT = (self.FAKE_CNF_SPAR1 - self.FAKE_CUR_SPAR1) * 2
-
-        self.assertEqual(DIFF_CNT, self.cw.create_amphora.call_count)
-
-    @mock.patch('octavia.db.api.get_session')
-    def test_spare_check_no_diff_count(self, session):
-        """When spare amphora count meets the requirement."""
-        session.return_value = session
-        self.CONF.config(group="house_keeping",
-                         spare_amphora_pool_size=self.FAKE_CNF_SPAR2)
-        self.amp_repo.get_spare_amphora_count.return_value = (
-            self.FAKE_CUR_SPAR2)
-        self.az_repo.get_all.return_value = [], None
-        self.spare_amp.spare_check()
-        self.assertTrue(self.amp_repo.get_spare_amphora_count.called)
-        DIFF_CNT = self.FAKE_CNF_SPAR2 - self.FAKE_CUR_SPAR2
-
-        self.assertEqual(0, DIFF_CNT)
-        self.assertEqual(DIFF_CNT, self.cw.create_amphora.call_count)
-
-    @mock.patch('octavia.db.repositories.SparesPoolRepository.get_for_update')
-    @mock.patch('octavia.db.api.get_session')
-    def test_spare_check_rollback(self, mock_session, mock_update):
-        """When spare amphora count meets the requirement."""
-        lock_session = mock.MagicMock()
-        session = mock.MagicMock()
-        mock_session.side_effect = [lock_session, session]
-        mock_update.side_effect = [Exception('boom')]
-#        self.CONF.config(group="house_keeping",
-#                         spare_amphora_pool_size=self.FAKE_CNF_SPAR2)
-#        self.amp_repo.get_spare_amphora_count.return_value = (
-#            self.FAKE_CUR_SPAR2)
-        self.spare_amp.spare_check()
-        lock_session.rollback.assert_called_once()
-
-
 class TestDatabaseCleanup(base.TestCase):
     FAKE_IP = "10.0.0.1"
     FAKE_UUID_1 = uuidutils.generate_uuid()
@@ -267,7 +168,7 @@ class TestCertRotation(base.TestCase):
                                                           amp_cert_mock
                                                           ):
         self.CONF.config(group="api_settings",
-                         default_provider_driver='amphora')
+                         default_provider_driver='amphorav1')
         amphora = mock.MagicMock()
         amphora.id = AMPHORA_ID
 
@@ -289,7 +190,7 @@ class TestCertRotation(base.TestCase):
                                                              amp_cert_mock
                                                              ):
         self.CONF.config(group="api_settings",
-                         default_provider_driver='amphora')
+                         default_provider_driver='amphorav1')
         amphora = mock.MagicMock()
         amphora.id = AMPHORA_ID
 
@@ -310,7 +211,7 @@ class TestCertRotation(base.TestCase):
                                                cert_exp_amp_mock,
                                                amp_cert_mock):
         self.CONF.config(group="api_settings",
-                         default_provider_driver='amphora')
+                         default_provider_driver='amphorav1')
 
         session.return_value = session
         cert_exp_amp_mock.return_value = None
@@ -326,7 +227,7 @@ class TestCertRotation(base.TestCase):
     def test_cert_rotation_expired_amphora_with_exception_amphorav2(
             self, session, cert_exp_amp_mock, amp_cert_mock):
         self.CONF.config(group="api_settings",
-                         default_provider_driver='amphorav2')
+                         default_provider_driver='amphora')
 
         amphora = mock.MagicMock()
         amphora.id = AMPHORA_ID
@@ -347,7 +248,7 @@ class TestCertRotation(base.TestCase):
     def test_cert_rotation_expired_amphora_without_exception_amphorav2(
             self, session, cert_exp_amp_mock, amp_cert_mock):
         self.CONF.config(group="api_settings",
-                         default_provider_driver='amphorav2')
+                         default_provider_driver='amphora')
         amphora = mock.MagicMock()
         amphora.id = AMPHORA_ID
 
@@ -367,7 +268,7 @@ class TestCertRotation(base.TestCase):
     def test_cert_rotation_non_expired_amphora_amphorav2(
             self, session, cert_exp_amp_mock, amp_cert_mock):
         self.CONF.config(group="api_settings",
-                         default_provider_driver='amphorav2')
+                         default_provider_driver='amphora')
         session.return_value = session
         cert_exp_amp_mock.return_value = None
         cr = house_keeping.CertRotation()
