@@ -1099,12 +1099,13 @@ class TestAllowedAddressPairsDriver(base.TestCase):
         lc_1 = data_models.ListenerCidr('l1', '10.0.101.0/24')
         lc_2 = data_models.ListenerCidr('l2', '10.0.102.0/24')
         lc_3 = data_models.ListenerCidr('l2', '10.0.103.0/24')
+        lc_4 = data_models.ListenerCidr('l2', '2001:0DB8::/32')
         listeners = [data_models.Listener(protocol_port=80, peer_port=1024,
                                           protocol=constants.PROTOCOL_TCP,
                                           allowed_cidrs=[lc_1]),
                      data_models.Listener(protocol_port=443, peer_port=1025,
                                           protocol=constants.PROTOCOL_TCP,
-                                          allowed_cidrs=[lc_2, lc_3]),
+                                          allowed_cidrs=[lc_2, lc_3, lc_4]),
                      data_models.Listener(protocol_port=50, peer_port=1026,
                                           protocol=constants.PROTOCOL_UDP)]
         vip = data_models.Vip(ip_address='10.0.0.2')
@@ -1183,7 +1184,18 @@ class TestAllowedAddressPairsDriver(base.TestCase):
                 'remote_ip_prefix': '10.0.103.0/24'
             }
         }
-        expected_create_rule_udp = {
+        expected_create_rule_5 = {
+            'security_group_rule': {
+                'security_group_id': 'secgrp-1',
+                'direction': 'ingress',
+                'protocol': 'tcp',
+                'port_range_min': 443,
+                'port_range_max': 443,
+                'ethertype': 'IPv6',
+                'remote_ip_prefix': '2001:0DB8::/32'
+            }
+        }
+        expected_create_rule_udp_1 = {
             'security_group_rule': {
                 'security_group_id': 'secgrp-1',
                 'direction': 'ingress',
@@ -1194,13 +1206,26 @@ class TestAllowedAddressPairsDriver(base.TestCase):
                 'remote_ip_prefix': None
             }
         }
+        expected_create_rule_udp_2 = {
+            'security_group_rule': {
+                'security_group_id': 'secgrp-1',
+                'direction': 'ingress',
+                'protocol': 'udp',
+                'port_range_min': 50,
+                'port_range_max': 50,
+                'ethertype': 'IPv6',
+                'remote_ip_prefix': None
+            }
+        }
 
         create_rule.assert_has_calls([mock.call(expected_create_rule_1),
                                       mock.call(expected_create_rule_udp_peer),
                                       mock.call(expected_create_rule_2),
                                       mock.call(expected_create_rule_3),
                                       mock.call(expected_create_rule_4),
-                                      mock.call(expected_create_rule_udp)],
+                                      mock.call(expected_create_rule_5),
+                                      mock.call(expected_create_rule_udp_1),
+                                      mock.call(expected_create_rule_udp_2)],
                                      any_order=True)
 
     def test_update_vip_when_protocol_and_peer_ports_overlap(self):
